import 'package:geolocator/geolocator.dart';
import '../../abg_utils.dart';

class AddressData{
  String id;
  String address;
  double lat;
  double lng;
  bool current;
  int type; // 1 home - 2 office - 3 other
  String name;
  String phone;

  AddressData({this.id = "", this.address = "", this.lat = 0,
    this.lng = 0, this.current = false, this.type = 1, this.name = "", this.phone = ""});

  factory AddressData.fromJson(Map<String, dynamic> data){
    return AddressData(
      id: (data["id"] != null) ? data["id"] : "",
      address: (data["address"] != null) ? data["address"] : "",
      lat: (data["lat"] != null) ? toDouble(data["lat"].toString()) : 0,
      lng: (data["lng"] != null) ? toDouble(data["lng"].toString()) : 0,
      current: (data["current"] != null) ? data["current"] : false,
      type: (data["type"] != null) ? data["type"] : 1,
      name: (data["name"] != null) ? data["name"] : "",
      phone: (data["phone"] != null) ? data["phone"] : "",
    );
  }

  Map<String, dynamic> toJson() => {
    'id': id,
    'address': address,
    'lat': lat,
    'lng': lng,
    'current': current,
    'type' : type,
    'name' : name,
    'phone' : phone,
  };
}


initProviderDistances() async {
  if (providers.isEmpty)
    return;
  if (userAccountData.userAddress.isEmpty)
    return;
  var _address = getCurrentAddress();
  if (_address.id.isNotEmpty) {
    for (var item in providers) {
      double _dist = double.infinity;
      for (var latLng in item.route) {
        double d = Geolocator.distanceBetween(_address.lat, _address.lng, latLng.latitude, latLng.longitude);
        if (d < _dist)
          _dist = d;
      }
      if (_dist == double.infinity)
        _dist = 0;
      item.distanceToUser = _dist;
    }
  }
//  parent.redraw();
}

AddressData getCurrentAddress(){
  for (var item in userAccountData.userAddress) {
    if (item.current)
      return item;
  }
  if (userAccountData.userAddress.isNotEmpty)
    return userAccountData.userAddress[0];
  return AddressData();
}

setCurrentAddress(String id){
  for (var item in userAccountData.userAddress) {
    item.current = false;
    if (item.id == id)
      item.current = true;
  }
  saveAddress();
  initProviderDistances();
}

